# Crea un disparador que actualizará el saldo del cliente, restándole el total del pedido.
#
# Esta actualización del saldo se debe realizar en el momento de la entrega del pedido, es decir, en el momento en el que la FechaEntrega pase del valor NULL a un valor no nulo.
#
# Para realizar este ejercicio puedes utilizar la función que calcula el total de un pedido, que se realizó en un ejercicio anterior.

DROP TRIGGER IF EXISTS afterUpdatePedidos;

DELIMITER $$

CREATE TRIGGER afterUpdatePedidos
AFTER UPDATE ON Pedidos
FOR EACH ROW

BEGIN
	DECLARE v_TotalPedido FLOAT;

	IF ISNULL(old.FechaEntrega) && NOT ISNULL(new.FechaEntrega) THEN
		SELECT totalPedido(new.IdPedido) INTO v_TotalPedido;

		UPDATE Clientes SET Saldo = IFNULL(Saldo, 0) - v_TotalPedido
			WHERE IdCliente = new.IdCliente;
	END IF;

END 

$$

DELIMITER ;


# Función que calcula el precio de un pedido con la utilización de un cursor
DROP function IF EXISTS `totalPedido`;

DELIMITER $$
USE `Neptuno`$$
CREATE DEFINER=`root`@`localhost` FUNCTION `totalPedido`(v_IdPedido INT) RETURNS float
BEGIN
  DECLARE done INT DEFAULT FALSE;
  DECLARE subtotal, totalPedido INT;
  DECLARE cursorDetallesPedido CURSOR FOR SELECT PrecioUnidad * Cantidad * (1-Descuento) FROM DetallesPedidos WHERE IdPedido = v_IdPedido;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursorDetallesPedido;
  SET totalPedido = 0;

  REPEAT
    FETCH cursorDetallesPedido INTO subtotal;
    IF NOT done THEN
	SET totalPedido = totalPedido + subtotal;
    END IF;
  UNTIL done END REPEAT;

  CLOSE cursorDetallesPedido ;
  RETURN totalPedido;
END$$

DELIMITER ;

# A la empresa le interesa que los datos referidos al destinatario de un pedido se rellenen automáticamente sin que los tenga que incluir el empleado que gestiona dicho pedido.
#
# Para ello, cada vez que se inserte un nuevo pedido, debemos buscar, en la tabla Clientes, el domicilio (dirección, ciudad, región, código postal,...) e incluiremos, con un disparador, dichos datos en el registro de pedidos a insertar.
#
# OPCIONAL:
#
# Haz que esos valores se incluyan únicamente en el caso en el que la dirección del destinatario no se haya especificado por el propio usuario.

DROP TRIGGER IF EXISTS beforeInsertPedidos;

DELIMITER $$

CREATE TRIGGER beforeInsertPedidos
BEFORE INSERT ON Pedidos
FOR EACH ROW

BEGIN
	DECLARE v_Destinatario,
			v_DireccionDestinatario,
			v_CiudadDestinatario,
			v_RegionDestinatario,
			v_CodPostalDestinatario,
			v_PaisDestinatario
				VARCHAR(100);

	SELECT NombreContacto, Direccion, Ciudad, Region, CodPostal, Pais
			INTO v_Destinatario, v_DireccionDestinatario, v_CiudadDestinatario, v_RegionDestinatario, v_CodPostalDestinatario, v_PaisDestinatario
	FROM Clientes
	WHERE IdCliente = new.IdCliente;

	IF ISNULL(new.Destinatario) THEN SET new.Destinatario = v_Destinatario; END IF;
	IF ISNULL(new.DireccionDestinatario) THEN SET new.DireccionDestinatario = v_DireccionDestinatario; END IF;
	IF ISNULL(new.CiudadDestinatario) THEN SET new.CiudadDestinatario = v_CiudadDestinatario; END IF;
	IF ISNULL(new.RegionDestinatario) THEN SET new.RegionDestinatario = v_RegionDestinatario; END IF;
	IF ISNULL(new.CodPostalDestinatario) THEN SET new.CodPostalDestinatario = v_CodPostalDestinatario; END IF;
	IF ISNULL(new.PaisDestinatario) THEN SET new.PaisDestinatario = v_PaisDestinatario; END IF;

END 

$$

DELIMITER ;


# Crear el disparador necesario para que, cuando se realiza un pedido,
# se decremente el número de existencias en almacén de cada uno
# de los productos que aparecen relacionados con dicho pedido.

# En el caso de que no existan suficientes existencias de un producto
# para cubrir el pedido, la cantidad pedida de dicho producto se debe reducir,
# haciéndola coincidir con las unidades existentes en el almacén.


DROP TRIGGER IF EXISTS beforeInsertDetallesPedidos;

DELIMITER $$

CREATE TRIGGER beforeInsertDetallesPedidos
BEFORE INSERT ON DetallesPedidos
FOR EACH ROW

BEGIN
	DECLARE v_pendientes INT;

	SELECT ABS(LEAST(UnidadesEnExistencia - new.Cantidad , 0))
			INTO v_pendientes
		FROM Productos
		WHERE IdProducto = new.IdProducto;

	SET new.Cantidad = new.Cantidad - v_pendientes;

END 

$$

DELIMITER ;


DROP TRIGGER IF EXISTS afterInsertDetallesPedidos;

DELIMITER $$

CREATE TRIGGER afterInsertDetallesPedidos
AFTER INSERT ON DetallesPedidos
FOR EACH ROW

BEGIN

	UPDATE Productos
		SET UnidadesEnExistencia = IFNULL(UnidadesEnExistencia, 0) - new.Cantidad
		WHERE IdProducto = new.IdProducto;

END 

$$

DELIMITER ;
